"""Handling configurations for the dataset acquisition."""

import logging
from pathlib import Path
from typing import Literal, cast

from pydantic import BaseModel, field_validator

from eurocropsml.settings import Settings
from eurocropsml.utils import _unzip_file

logger = logging.getLogger(__name__)


class CollectorConfig(BaseModel):
    """Country-specific configuration for acquiring EuroCrops reflectance data."""

    country: str
    year: int
    months: tuple[int, int] = (1, 12)
    processing_level: Literal["L1C", "L2A"] = "L1C"
    max_cloud_cover: int = 100
    bands: list[str] = [
        "01",
        "02",
        "03",
        "04",
        "05",
        "06",
        "07",
        "08",
        "8A",
        "09",
        "10",
        "11",
        "12",
    ]
    max_requested_products: int = 1000
    country_code: str | None = None
    ec_filename: str | None = None
    shapefile_folder: Path | None = None
    shapefile: Path | None = None
    polygon: str | None = None
    parcel_id_name: str | None = None

    def post_init(self, vector_data_dir: Path) -> None:
        """Make dynamic config based on initialized params."""
        if len(self.months) != 2 or not (1 <= self.months[0] <= self.months[1] <= 12):
            logger.info(
                "months should be a tuple of two integer values between 1 and 12,"
                " corresponding to the months. Using months 1 to 12."
            )
            self.months = (1, 12)

        if self.max_cloud_cover > 100:
            logger.info("Maximum cloud cover set to 100.")
            self.max_cloud_cover = 100

        if self.max_requested_products > 1000:
            logger.info("Maximum requested products set to 1000.")
            self.max_requested_products = 1000

        eurocrops_config: EuroCropsCountryConfig = EuroCropsCountryConfig()

        if self.country not in eurocrops_config.countries:
            raise ValueError(
                f"{self.country} is not a valid country. "
                f"Please use one of the following {', '.join(eurocrops_config.countries.keys())}"
            )
        else:
            eurocrops_countries = eurocrops_config.countries
            country_years: set[int] = set(cast(list, eurocrops_countries[self.country]["years"]))
            if self.year not in country_years:
                raise ValueError(
                    f"Year {self.year} not available. Please select one of the following: "
                    f"{', '.join(str(item) for item in country_years)}."
                )
            self.country_code = cast(str, eurocrops_countries[self.country]["country_code"])
            self.ec_filename = cast(str, eurocrops_countries[self.country]["ec_zipfolder"])

            if self.country_code == "ES" and self.year == 2021:
                filename = f"{self.ec_filename}_2020"
            elif self.country_code == "PT" and self.year == 2021:
                filename = self.ec_filename
            else:
                filename = f"{self.ec_filename}_{self.year}"

            # check if folder exists
            shapefile_folder_list = [
                item
                for item in vector_data_dir.iterdir()
                if item.is_dir() and item.name == filename
            ]

            if shapefile_folder_list == []:
                # check if zip-file exists
                zip_file = [item for item in vector_data_dir.glob("*.zip") if item.stem == filename]
                if len(zip_file) == 1:
                    unzip_folder = zip_file[0].parent
                    if self.country_code == "PT":
                        unzip_folder = unzip_folder.joinpath(zip_file[0].stem)
                    _unzip_file(zip_file[0], unzip_folder)
                    shapefile_folder_list = [
                        item
                        for item in vector_data_dir.iterdir()
                        if item.is_dir() and item.name == filename
                    ]
                else:
                    raise ValueError(
                        "Vector data does not exist. "
                        f"Please first download the EuroCrops reference data {filename}.zip."
                    )

            self.shapefile_folder = cast(Path, shapefile_folder_list[0])
            # there should at max be one folder
            for file in self.shapefile_folder.iterdir():
                if file.is_dir():
                    self.shapefile = cast(
                        Path, [file for file in file.iterdir() if file.suffix == ".shp"][0]
                    )
                else:
                    self.shapefile = cast(
                        Path,
                        [file for file in self.shapefile_folder.iterdir() if file.suffix == ".shp"][
                            0
                        ],
                    )
                    break

            self.polygon = eurocrops_config.polygon[self.country]
            self.parcel_id_name = eurocrops_config.parcel_ids[self.country]


class AcquisitionConfig(BaseModel):
    """Configuration for acquiring EuroCrops reflectance data."""

    raw_data_dir: Path
    output_dir: Path
    eodata_dir: str | None = None
    local_dir: Path | None = None
    country_config: CollectorConfig
    workers: int
    chunk_size: int
    multiplier: int

    @field_validator("raw_data_dir", "output_dir", "local_dir")
    @classmethod
    @classmethod
    def relative_path(cls, v: Path) -> Path:
        """Interpret relative paths w.r.t. the project root."""
        if not v.is_absolute():
            v = Settings().data_dir.joinpath(v)
        return v


class Config(BaseModel):
    """Configuration for acquiring EuroCrops reflectance data."""

    cfg: AcquisitionConfig


class EuroCropsCountryConfig(BaseModel):
    """Configuration for EuroCrops vector data."""

    countries: dict[str, dict[str, str | list[str] | list[int]]] = {
        "Austria": {"country_code": "AT", "ec_zipfolder": "AT", "years": [2021]},
        "Czechia": {"country_code": "CZ", "ec_zipfolder": "CZ", "years": [2023]},
        "Belgium VLG": {"country_code": "BE", "ec_zipfolder": "BE_VLG", "years": [2021]},
        "Croatia": {"country_code": "HR", "ec_zipfolder": "HR", "years": [2020]},
        "Denmark": {"country_code": "DK", "ec_zipfolder": "DK", "years": [2019]},
        "Estonia": {"country_code": "EE", "ec_zipfolder": "EE", "years": [2021]},
        "France": {"country_code": "FR", "ec_zipfolder": "FR", "years": [2018]},
        "Germany LS": {"country_code": "DE", "ec_zipfolder": "DE_LS", "years": [2021]},
        "Germany NRW": {"country_code": "DE", "ec_zipfolder": "DE_NRW", "years": [2021]},
        "Germany BB": {"country_code": "DE", "ec_zipfolder": "DE_BB", "years": [2023]},
        "Latvia": {"country_code": "LV", "ec_zipfolder": "LV", "years": [2021]},
        "Lithuania": {"country_code": "LT", "ec_zipfolder": "LT", "years": [2021]},
        "Netherlands": {"country_code": "NL", "ec_zipfolder": "NL", "years": [2020]},
        "Portugal": {"country_code": "PT", "ec_zipfolder": "PT", "years": [2021]},
        "Slovakia": {"country_code": "SK", "ec_zipfolder": "SK", "years": [2021]},
        "Slovenia": {"country_code": "SI", "ec_zipfolder": "SL", "years": [2021]},
        "Spain NA": {"country_code": "ES", "ec_zipfolder": "ES_NA", "years": [2021]},
        "Sweden": {"country_code": "SE", "ec_zipfolder": "SE", "years": [2021]},
    }

    # If no unique identifier exists (empty string), one will be created.
    parcel_ids: dict[str, str] = {
        "Austria": "fid",
        "Belgium": "fid",
        "Croatia": "id",
        "Czechia": "FID",
        "Denmark": "ID",
        "Estonia": "pollu_id",
        "France": "ID_PARCEL",
        "Germany LS": "",  # no unique identifier
        "Germany NRW": "ID",
        "Germany BB": "",
        "Latvia": "PARCEL_ID",
        "Lithuania": "parcel_id",
        "Netherlands": "objectid",
        "Portugal": "OSA_ID",
        "Slovakia": "",  # no unique identifier
        "Slovenia": "ID",
        "Spain NA": "REFSIGPAC",
        "Sweden": "ID",
    }

    polygon: dict[str, str] = {
        "Austria": """\
POLYGON((9.7084722891628+47.55044479889082%2C9.51807439566023+47.54572872207399%2C9.549\
51624963313+47.47847851645747%2C9.617640266574417+47.43241556005398%2C9.638601502556352+47.38394689\
0057956%2C9.565237176619581+47.3425359053368%2C9.490576453948579+47.28902206763439%2C9.569675062317\
899+47.16624531356172%2C9.596041265107674+47.10987758104221%2C9.577208263114978+47.04831735252546%2\
C9.874769694599568+46.989255386794355%2C9.848403491809794+46.917265676988535%2C10.108298919308991+4\
6.821987869962584%2C10.251429734453476+46.8503313715008%2C10.387027348800885+46.95327263682003%2C10\
.454826155974587+46.922411010707606%2C10.420033046404722+46.83640157932675%2C10.529264457962359+46.\
81320654136289%2C10.672395273106844+46.846707272677634%2C10.725127678686391+46.83382484710813%2C10.\
649795670715609+46.77968492850184%2C10.853192092236721+46.73581746301457%2C11.033988911366599+46.74\
87233660014%2C11.128153921330075+46.810628697134575%2C11.18465292730816+46.92393696540125%2C11.6215\
78573538697+46.975361661867595%2C11.719510183900713+46.94965548967201%2C12.175268832123944+47.05753\
841586767%2C12.096170223754623+47.006192764834594%2C12.130069627341475+46.8879102785082%2C12.246834\
239696186+46.83382484710813%2C12.252361105191218+46.76592427471701%2C12.327693113162002+46.74786091\
167732%2C12.41055832192986+46.64711131975997%2C12.806051363776469+46.605723770827524%2C12.806051363\
776469+46.57207308640346%2C13.141278799246447+46.57466234229864%2C13.186478004028915+46.55135458883\
4954%2C13.37857462435441+46.52544534050995%2C13.623403650259451+46.507301510621204%2C13.90083604520\
561+46.48915162256708%2C14.100465866328179+46.452833671161414%2C14.168264673501884+46.4061035513211\
1%2C14.405560498609848+46.42687743961014%2C14.495958908174785+46.39831130286663%2C14.54869131375433\
6+46.35413420567568%2C14.61649012092804+46.372329103374994%2C14.65038952451489+46.416491484714015%2\
C14.77845393806522+46.48396482751346%2C14.880152148825774+46.49693088784795%2C14.906518351615548+46\
.603135998336114%2C15.015749763173183+46.57725147457339%2C15.05624071745748+46.63935356466823%2C15.\
420706696722245+46.627299213958395%2C15.457128194572409+46.60017431620844%2C15.570621170292695+46.6\
19665647902934%2C15.590264185321205+46.66911214325302%2C15.887091967974257+46.70205136611429%2C16.0\
3987097375157+46.619665647902934%2C16.09661746161171+46.67060981638858%2C16.061696546005468+46.8201\
67580517904%2C16.349794099756963+46.96931048563431%2C16.607335852352996+46.981224000343445%2C16.528\
76379223895+47.2219017330722%2C16.533128906689733+47.364012277812435%2C16.755749743679523+47.470344\
61916178%2C16.672812569114697+47.647088117626055%2C17.170435616503642+47.67942642732518%2C17.139879\
815348177+47.878891386378456%2C17.20972164656066+48.00170813092336%2C17.1355147008974+48.1358870096\
9538%2C17.035117068529452+48.23774533511755%2C16.905067880137246+48.42364669973054%2C17.04140002746\
352+48.54037256869589%2C16.990275472216165+48.76178644344873%2C16.746013708256594+48.78424865444145\
6%2C16.62104257320751+48.85531269862628%2C16.416544352218104+48.85531269862628%2C16.348378278554964\
+48.76178644344873%2C16.115477526872585+48.79921787891507%2C15.780327664695495+48.91881092785948%2C\
15.712161591032359+48.911344732856776%2C15.308845655192135+49.04184207478667%2C15.178194014004456+4\
9.02322051208273%2C14.928251743906289+49.079064291233465%2C14.922571237767695+48.825403275807474%2C\
14.712392510639692+48.82166334211979%2C14.661267955392338+48.6568298912255%2C14.439728215987145+48.\
675588189553224%2C14.320437587076656+48.60802563581217%2C14.093217341532869+48.634310716224036%2C14\
.15570290905741+48.69808893141834%2C13.855995032940733+48.789571354396514%2C13.775869653757503+48.7\
63970297940716%2C13.78558182093123+48.6870887810006%2C13.800150071691817+48.610089710718285%2C13.71\
2740567128293+48.55226323925339%2C13.484504638545758+48.616510790785696%2C13.389811008601944+48.577\
97206055639%2C13.380098841428218+48.397735805478504%2C13.26355283534352+48.31545163174641%2C12.9891\
84112685795+48.28476276251743%2C12.859613318604735+48.22787328854807%2C12.711151871021515+48.128877\
19216724%2C12.762940748085429+48.018144081739706%2C12.942475521906996+47.85623102851258%2C12.863065\
910408995+47.74955488001453%2C12.91830737927717+47.68218992402788%2C13.052958459643344+47.691486820\
12977%2C13.011527357992215+47.526220535568285%2C12.907949603864388+47.52855185581046%2C12.838897767\
779168+47.58214362375253%2C12.835445175974908+47.64033338409649%2C12.707699279217255+47.70775240103\
7396%2C12.5005437709616+47.69148682012977%2C12.448754893897686+47.714721810857014%2C12.283030487293\
164+47.707752401037396%2C12.324461588944295+47.76348158411031%2C12.189810508578121+47.7658023390599\
2%2C12.065517203624728+47.693810785237616%2C12.148379406926988+47.63800703676546%2C11.7582365330455\
07+47.61008278899604%2C11.585606942832461+47.60309939625023%2C11.492386964117417+47.542537601267725\
%2C11.402619577206634+47.53787610058569%2C11.27832627225324+47.444559040680446%2C11.203250067869508\
+47.44297529168546%2C11.005363216970055+47.41260047603521%2C10.95214994613995+47.49019016578606%2C1\
0.905588334163607+47.55644419483113%2C10.785858474795871+47.53511728595245%2C10.682757762562543+47.\
571031290929994%2C10.612351974495121+47.57637826356236%2C10.570779106659103+47.567402986160744%2C10\
.547498300670933+47.54944781662806%2C10.487633370987064+47.55842617053037%2C10.52089166525588+47.58\
75951972464%2C10.484307541560181+47.59993104845651%2C10.401161805888144+47.593202762834494%2C10.429\
431356016636+47.55618172626947%2C10.411139294168787+47.49554538710035%2C10.451049247291367+47.43371\
409230923%2C10.319678984929544+47.31997991775154%2C10.224892846263419+47.28502220086537%2C10.244847\
82282471+47.362799779951246%2C10.256488225818794+47.39545473139495%2C10.219904102123097+47.41346259\
9159146%2C10.133432537024175+47.39432903526057%2C10.12012921931665+47.447210759678455%2C10.04529805\
7211813+47.50340960761915%2C10.005388104089235+47.49891591156202%2C9.970466895106979+47.56179265665\
5655%2C9.842422462172038+47.55618172626947%2C9.837433718031717+47.60890075014379%2C9.78089461777473\
+47.6145060326663%2C9.7084722891628+47.55044479889082))""",
        "Belgium VLG": """\
POLYGON((3.3414193094486637+51.39310226216443%2C3.2447547874575884+51.37080609247337%2C3.1564958760\
74433+51.38392280183069%2C2.5050610539606666+51.10372374439993%2C2.555494717608184+50.9557037669534\
34%2C2.561798925564123+50.83110766520252%2C2.721505527114595+50.72480576842784%2C2.849691088885369+\
50.66875253402608%2C2.969471040048223+50.66342473213979%2C3.1270762389467146+50.75391516170049%2C3.\
175408499942252+50.68739507999783%2C3.335518823424784+50.68567293698359%2C3.4470330494263135+50.732\
31930609836%2C3.5339936843816346+50.69215437493244%2C3.660853904787043+50.701874852962476%2C3.72939\
9346457708+50.74526839470059%2C3.7733911970821645+50.713536767405344%2C3.8770514211344977+50.669381\
62966301%2C4.0448342932835875+50.67003003878668%2C4.210571032845493+50.68494097698024%2C4.307762330\
736734+50.6577087339906%2C4.462245341069128+50.72575972500812%2C4.541021445675712+50.69466294979165\
%2C4.665835533493938+50.73353069589936%2C4.76404989767877+50.777541851481544%2C4.867379593331565+50\
.734825732390874%2C4.960478626048437+50.71021391587169%2C5.188622409519455+50.67975510099791%2C5.38\
3005005301938+50.70762244632803%2C5.522142021230453+50.71733971849986%2C5.651048374223047+50.767837\
0659209%2C5.712432351838569+50.80470453984054%2C5.723686081068082+50.88674333337784%2C5.79120845644\
5159+50.955750687119775%2C5.813715914904183+51.054247005878835%2C5.888399754336403+51.1352046403735\
3%2C5.869984561051745+51.178837185937596%2C5.728801412536047+51.212816295394475%2C5.605010391011413\
+51.23780378832694%2C5.563064672974141+51.30181230776154%2C5.4750809717252285+51.32803003294677%2C5\
.406535530054564+51.27877951174986%2C5.271490779300419+51.28773808305223%2C5.284790641117114+51.323\
55489657033%2C5.173276415115586+51.36253783674499%2C5.136446028546273+51.398934555607326%2C5.152815\
089243745+51.44741849974119%2C5.088361912747449+51.49521408094026%2C4.982986084507472+51.4933032190\
3267%2C4.91034837766244+51.42701047670556%2C4.848964400046919+51.43466455386158%2C4.873517991093127\
+51.494577135871225%2C4.743753872054266+51.51940618627799%2C4.632239646052738+51.44677566600723%2C4\
.56267113808848+51.44677566600723%2C4.581086331373136+51.500304106188366%2C4.430695586215111+51.485\
653753787346%2C4.341688818672606+51.44996364087416%2C4.371357741186775+51.391907653828326%2C4.19027\
5007220988+51.38807719485155%2C4.191298073514582+51.34783801278596%2C4.0480687924117005+51.26469298\
4492314%2C3.9212085720062912+51.232674009223416%2C3.8189019426470905+51.23779854344062%2C3.83527100\
33445625+51.279414227508056%2C3.6152723128485604+51.31919832002322%2C3.484319827268783+51.294255054\
00192%2C3.4495355732866546+51.260336061486754%2C3.39633612601987+51.31536177689125%2C3.444420241818\
695+51.340932670845746%2C3.3414193094486637+51.39310226216443))""",
        "Croatia": """POLYGON((13.553432705775174+45.53451891961268%2C13.461940405701037+45.5013599\
38874316%2C13.556587612674283+45.25981774617151%2C13.600756309261799+45.05291545182672%2C13.7805860\
02510962+44.80498412786051%2C13.925711719869938+44.740032953808765%2C14.048753088935158+44.79155194\
708838%2C14.083457064825344+44.93466733209266%2C14.219118061486995+44.974854431798065%2C14.22227296\
8386104+45.09747397598471%2C14.291680920166483+44.97039058825811%2C14.275906385670945+44.9100946947\
8084%2C14.310610361561135+44.83407645921028%2C14.178104271798592+44.60991146841138%2C14.28315740223\
3622+44.490177925494464%2C15.437002195239339+43.623957314901475%2C15.809210192983114+43.59078691255\
28%2C16.084071483624673+43.372652306578175%2C15.671779547662334+43.00734498214587%2C15.992451053410\
822+42.921442625632864%2C16.479184588921918+43.04920504008069%2C16.62806778801943+42.81442109321222\
%2C16.410469266261526+42.7724012182382%2C16.487774004254465+42.69879779250172%2C16.911889581097018+\
42.68637577574657%2C17.3380886444563+42.73403183801338%2C18.11945359394831+42.586406344204846%2C18.\
456089086021947+42.40423238655583%2C18.516913216037164+42.560487519595625%2C18.33320286409571+42.67\
182336790964%2C17.760458825690005+42.98487606133688%2C17.609167947620573+43.19400983733817%2C17.247\
150489383003+43.547506285230014%2C16.274566273222366+44.244444462465566%2C16.112468903862265+44.675\
43618271051%2C15.863919604176768+44.809759002786734%2C15.863919604176768+45.15753773171062%2C15.977\
387762728844+45.161347756396566%2C16.20432407983299+44.951418060234005%2C16.436663642582477+44.9705\
3442488229%2C16.571744783715896+45.172776301829515%2C16.885133031145436+45.146106129025156%2C17.565\
941982457883+45.046936189748294%2C17.916282938049346+44.99947104022283%2C18.03616340254349+45.04183\
989952122%2C18.37082969925631+45.05242721538522%2C18.675525879845587+45.01006619442509%2C18.7454561\
50800507+44.81551034536531%2C19.04016229268194+44.811966869865216%2C19.249953105546695+44.953536061\
25056%2C19.195007892653546+45.10885980486708%2C19.5596443054899+45.16171473525216%2C19.514689131304\
596+45.26727763008083%2C19.085117466867246+45.39018668245586%2C19.180022834591778+45.56181130999144\
%2C18.905296770126032+45.95907883248145%2C18.63556572501421+45.969495045309515%2C18.420779892795537\
+45.78170354723764%2C17.876322783217965+45.8234898349732%2C17.41678100265708+46.02501510471586%2C17\
.246950344623713+46.19124107721251%2C16.742453389877525+46.48780921351903%2C16.34285184156371+46.58\
401921032214%2C16.168026164176418+46.4774908892353%2C16.22297137706957+46.42931283303702%2C16.03815\
566097443+46.432755536217826%2C15.518673648166471+46.22580839666006%2C15.618574035244924+46.0215467\
3298534%2C15.60358897718316+45.893064003170224%2C15.184007351453655+45.73639970059%2C15.25838732878\
3801+45.509742023013956%2C15.143501883643577+45.50274068857337%2C14.908735974009215+45.583203461501\
34%2C14.7738704514533+45.58320346150134%2C14.614029832127775+45.708914012629066%2C14.44919419344832\
8+45.621644949468106%2C14.409234038616948+45.537738652924304%2C14.124517935443354+45.52724154942174\
6%2C13.899742064516834+45.565721350459796%2C13.553432705775174+45.53451891961268))""",
        "Czechia": """POLYGON((12.15569032405676+50.35290217514961%2C11.99915302271018+50.271680516\
69841%2C12.423108213857171+49.8529768970871%2C12.293146858130852+49.70008182249725%2C13.03060250397\
8914+49.01140337648974%2C14.0881660022105+48.50736374192746%2C14.941126749215483+48.4779225496803%2\
C15.181021959310637+48.830089147882916%2C16.655933251006758+48.6014606971431%2C16.89340850786027+48\
.55227470686569%2C17.27324675321363+48.737772184456986%2C17.61179823276771+48.77043622204633%2C18.2\
39357072916743+48.97681546227443%2C18.27238648555617+49.252460887738266%2C18.875173266225634+49.403\
14464270418%2C19.040320329422745+49.65504192981686%2C18.53662178667155+50.06492241923095%2C17.86777\
618072324+50.13378012549913%2C17.82648941492396+50.35029880688268%2C16.892359842199664+50.550363571\
00613%2C16.78501425112154+50.27674424420226%2C16.429948065247746+50.44005367668575%2C16.62812454108\
4283+50.655181218163904%2C16.23177158941121+50.75454180826222%2C15.94276422881626+50.75976548118353\
5%2C15.54641127714319+50.84326497048676%2C15.183087738109537+51.08249541350304%2C14.844536258555456\
+51.04617023111524%2C14.753705373797045+50.905791691991624%2C14.481212719521807+51.16541738980271%2\
C14.084859767848734+51.01501167004949%2C14.167433299447291+50.905791691991624%2C12.15569032405676+5\
0.35290217514961))""",
        "Denmark": """\
POLYGON((8.040205882308747+55.09442219525249%2C8.284424793091661+55.06438234725\
118%2C8.441855165331885+55.052822551933986%2C8.530662041980216+54.98223549091932%2C8.53469871819150\
5+54.909203586583914%2C8.835431095932446+54.8825080755486%2C9.212860321687856+54.85230922998733%2C9\
.214887928854838+54.79601982811158%2C9.390483344045856+54.78787415840705%2C9.576170449765096+54.849\
50769633147%2C9.745710850639185+54.812306244442055%2C9.903141222879409+54.822772617763974%2C10.0689\
65682559796+54.85503230367749%2C10.273221810270703+54.877575487624966%2C10.452295675661091+54.79216\
695050988%2C10.614581366171128+54.8131329426792%2C10.63976362849165+54.70981067491289%2C10.73489661\
9480292+54.682321348729175%2C10.953142892924827+54.78410021095277%2C11.52531263181186+54.5646192495\
2387%2C12.03543940913737+54.546957900037%2C12.00005116777384+54.672996193658236%2C12.23111715037245\
5+54.86169527520363%2C12.429173706885551+54.93934212695561%2C12.54620712664329+54.913476478257365%2\
C15.131871903197244+54.975105308482085%2C15.236901895287522+55.13838963403256%2C14.768768216256564+\
55.324917438671605%2C14.690745936418072+55.242884536660824%2C14.663738224166286+55.109219246797096%\
2C14.639731368831365+54.99060291537896%2C12.587145237695628+54.93892095502693%2C12.54213238394265+5\
5.02158018376332%2C12.4431041056861+55.04221840690451%2C12.326070685928363+55.04221840690451%2C12.2\
42046692256139+55.1263808785848%2C12.152020984750184+55.15553873334241%2C12.281057832175385+55.2240\
61448488925%2C12.410094679600585+55.22063811165654%2C12.521126385524594+55.28734006751347%2C12.4250\
9896418491+55.401669407109694%2C12.266053547591058+55.42892313892466%2C12.247976645771729+55.498878\
26815555%2C12.437030631534231+55.585470269420256%2C12.511184556494236+55.58210203433447%2C12.588930\
30668385+55.54606598264195%2C12.734225643103782+55.59146599376581%2C12.861677692594954+55.600107665\
95465%2C12.873148377049159+55.66414068261474%2C12.789030024384985+55.74815980246822%2C12.6104020903\
4169+55.73394764533373%2C12.61677469281625+55.7791303682402%2C12.54157798361646+55.91865015641454%2\
C12.640990582219573+56.03204208440542%2C12.6433469738155+56.055642194360246%2C12.520993006303977+56\
.097608245474106%2C12.387168354338248+56.10898099323987%2C12.32344232959266+56.14023872877161%2C12.\
157754665254139+56.11750834935731%2C11.842948103010949+55.98155680572964%2C11.73621927892091+56.209\
86829201408%2C11.653950564420656+56.20333145631267%2C11.697043700587457+55.96835863789428%2C11.5795\
16965587095+55.97603150026214%2C11.505083366753531+55.97603150026214%2C11.375803958253131+55.985894\
37308814%2C11.266010592019962+56.02197271075309%2C11.071022230217249+55.92481187745949%2C10.9703140\
72501633+55.86195151897519%2C10.833472890806881+55.75745055897528%2C10.714175450355045+55.666524039\
74709%2C10.882595366287049+55.91901259409269%2C11.335223890354301+56.450132881275266%2C11.742238687\
189976+56.743742625599594%2C11.622941246738138+56.80526477886551%2C10.558552274025164+56.5421698584\
3573%2C10.351361977406128+56.953424735511476%2C10.599990333348972+57.19442970956936%2C11.0392337621\
81327+57.169725656303086%2C11.354163013042262+57.27964396725727%2C11.196698387611795+57.37136495267\
484%2C10.628996974875637+57.2706833177468%2C10.705316570742633+57.49920386602511%2C10.6721661232835\
85+57.778668210060914%2C10.083745680885523+57.64142283264951%2C9.864123966469347+57.59925955853356%\
2C9.524331880014127+57.27811266907486%2C9.19697121135605+57.16594305318645%2C8.981493302872254+57.2\
1533970145276%2C8.764003156197896+57.12953785575087%2C8.523662412119815+57.161010687527096%2C8.0586\
58895823555+56.68258982994115%2C8.050371283958794+56.13004492287635%2C8.077468008485594+55.68704266\
455495%2C7.957297636446553+55.525531995857534%2C8.23493263391606+55.41748841473631%2C8.313664946631\
295+55.11291448617507%2C8.040205882308747+55.09442219525249))""",
        "Estonia": """\
POLYGON((27.381286918305708+57.510237039736495%2C27.888631575610912+57.88729405\
7120585%2C27.643706568635984+58.10055669729141%2C27.503749421793174+58.232818021983746%2C27.5737279\
95214583+58.389049394737924%2C27.276319058173595+58.48062817819175%2C27.177182745826606+58.73578352\
603255%2C26.99640476448797+58.799281737284076%2C26.98474166891774+58.856632556540205%2C27.171351198\
041485+58.96503725225645%2C27.79532681104903+58.98006656914026%2C28.273513729428647+59.392295724298\
04%2C28.040251818023954+59.507895863037106%2C27.91778931453649+59.42197446446718%2C26.8447845220749\
24+59.49309741000005%2C26.588196419529762+59.58474327493866%2C25.63765413055565+59.72909514648006%2\
C25.456876149217017+59.67025105073344%2C24.483007669102435+59.649631176355314%2C24.290566592193567+\
59.44273411474862%2C23.812379673813954+59.35664690198857%2C23.240887990872462+59.30905683298025%2C2\
2.511944517732804+59.10006534401555%2C22.389482014245342+58.986076460907924%2C22.010431408212725+58\
.95601651585241%2C21.99876831264249+58.90184251973281%2C22.389482014245342+58.820422027164994%2C22.\
447797492096516+58.611476047215746%2C22.068746886063895+58.52328345060573%2C21.748011757882445+58.5\
5980387860765%2C21.71885401895686+58.297232164681276%2C22.12123081612995+58.13751631320511%2C21.899\
632000295497+57.93685940590322%2C22.039589147138308+57.881093579343684%2C22.24952486740253+57.97398\
8555311394%2C22.296177249683467+58.11288082844001%2C22.58775463893933+58.1098001950142%2C23.5499600\
23483674+58.508055373793184%2C23.61410704911997+58.41960213092952%2C23.1534147740957+57.76619178799\
5695%2C24.27890349662333+57.89659276950732%2C24.44801838239173+57.84076441000545%2C25.1711303077462\
7+58.02961023748156%2C25.270266620093263+57.961616442972286%2C25.946726163166854+57.812817740035115\
%2C25.98171544987756+57.73818729900242%2C26.51821784610835+57.45381009008352%2C27.381286918305708+5\
7.510237039736495))""",
        "France": """\
POLYGON((-5.3543515022541746+48.53918482054581%2C-5.067309141312095+48.045832126\
0318%2C-4.242062353603614+47.66060528391762%2C-3.0101722212271858+47.15064872298916%2C-2.5317682863\
237194+47.04480490292738%2C-2.460007696088199+46.58645606940297%2C-1.395558940927985+45.78328007632\
865%2C-1.759995061535031+43.64560768040565%2C-1.9309533799675536+43.48379690833485%2C-1.75531231193\
58878+43.21009468229175%2C-1.4855778146015435+43.19637717836318%2C-1.5733983486173766+43.0223551924\
7404%2C-1.3726656994383293+42.96729810089897%2C-1.2346620031277349+43.01318243605459%2C-0.826923809\
4827959+42.921379513486585%2C-0.5618482119951749+42.74391020251937%2C-0.30626381815621206+42.779263\
84875755%2C-0.07475323644076484+42.67002073151514%2C0.1919435223477181+42.664573438732134%2C0.20305\
588729723836+42.699972308934946%2C0.2956589285432394+42.64005471416144%2C0.3734454831898802+42.6781\
9077565002%2C0.44752791618668114+42.66184961333548%2C0.6882958234262837+42.672744198899665%2C0.7068\
16431675484+42.81963419506684%2C1.0957492049086883+42.735351009258665%2C1.1179739348077287+42.67546\
75469449%2C1.3513335987476514+42.697250034984165%2C1.3920789368958917+42.6018952643752%2C1.46986549\
15425324+42.56371242888429%2C1.5439479245393335+42.62370352797606%2C1.7032251554824553+42.569168551\
347104%2C1.6958169121827753+42.47088534095144%2C1.8513900214760568+42.45722266905739%2C1.9995548874\
69659+42.309475853768845%2C2.1810568483118207+42.37792037955765%2C2.306996984406382+42.394335977705\
28%2C2.407008268952063+42.3833927225497%2C2.467967469286795+42.31631197280862%2C2.7003782626001605+\
42.32713647234755%2C2.724168343805466+42.3933958945687%2C2.870568843530421+42.43527903288668%2C3.01\
69693432553752+42.45148435100566%2C3.088339586871291+42.40961204036168%2C3.2388299638016416+42.4144\
82417531076%2C3.1124391942572154+42.60356572904922%2C3.1087218186823793+43.037100740202334%2C3.4061\
118646692643+43.25137411125391%2C3.5659590143872157+43.248666483704085%2C3.7258061641051667+43.3730\
927598433%2C4.067804716990086+43.510745824440676%2C4.227651866708036+43.416311750416554%2C4.8596057\
14430168+43.286562315670125%2C4.9699942564493576+43.360231859244806%2C5.0480591435209154+43.3007428\
1683818%2C5.246938736774646+43.31291577026897%2C5.28039511694817+43.18294526585379%2C5.462546520115\
137+43.086642869012934%2C5.542470094974113+43.15990075441104%2C5.738063319202499+43.10098585412163%\
2C5.782671826100532+43.04939350539547%2C5.879323591046269+43.012709389948526%2C6.000138297228442+43\
.062974650444005%2C6.155087755880196+42.94534366993298%2C6.433890923992902+42.95622724207391%2C6.71\
2694092105607+43.04594327303229%2C6.768454725728147+43.317009847321%2C7.2331266725826575+43.5437754\
72402416%2C7.3793905596086615+43.70905425350389%2C7.469389521718666+43.735229796066875%2C7.55609583\
8873427+43.75267380410503%2C7.53240561372737+43.86992259047824%2C7.60264870610591+43.89840038995726\
%2C7.6048438027427405+43.942672129484464%2C7.67289179848445+43.95531514708557%2C7.692647668215916+4\
4.021646866508036%2C7.751915277410308+44.04531880118827%2C7.725574117768357+44.11312594052464%2C7.6\
86062378305427+44.13676131157081%2C7.705818248036892+44.18242954703581%2C7.60703889937957+44.206037\
17775481%2C7.6048438027427405+44.16983494371459%2C7.380943945786142+44.14306248005997%2C7.312895950\
044431+44.1666858725705%2C7.277774403855161+44.1666858725705%2C7.258018534123697+44.19029980808085%\
2C7.13728821909808+44.22491647303451%2C7.060459836809051+44.248507080335486%2C7.020948097346122+44.\
251651780061394%2C7.018753000709295+44.29843243987088%2C6.926558941962461+44.38791437962172%2C6.955\
095198241242+44.413007566550306%2C6.985826551156855+44.430252889624484%2C6.8826570092258725+44.5132\
7268983002%2C6.992411841067341+44.57585094350489%2C7.003387324251489+44.669592190545444%2C7.1175323\
49366616+44.65397916073297%2C7.094041425811432+44.72085350381903%2C7.047944396438014+44.74580350526\
2444%2C7.039164009890698+44.82058892471767%2C7.02818852670655+44.85483319839102%2C6.92282388813874+\
44.88905712065943%2C6.872336665491663+44.87350331526466%2C6.780142606744829+44.912379944639156%2C6.\
7559965437397045+45.02885214985062%2C6.683558354724336+45.03971082097516%2C6.674777968177017+45.107\
9181464269%2C6.760386737013365+45.14199128183165%2C6.861361182307515+45.118761806148115%2C6.9228238\
8813874+45.1218596162029%2C7.03038362334338+45.200796944270564%2C7.129162972000701+45.1946097338092\
7%2C7.174760966391395+45.2792937595903%2C7.1528100000231+45.317895197896064%2C7.212077609217494+45.\
416595133079255%2C7.137444323565296+45.4474035451029%2C7.1396394202021245+45.48127337411805%2C7.016\
714008539679+45.50896991691778%2C7.032079684997485+45.65031729606224%2C6.972812075803091+45.6779305\
9920277%2C6.926715046429674+45.67486312700905%2C6.880618017056257+45.710128905352406%2C6.8367160843\
19669+45.725454916481624%2C6.836716084319669+45.81579282099884%2C6.925621859112528+45.8127268357522\
15%2C7.050742367411804+45.893757547944205%2C7.0918704712788605+45.92482795687724%2C6.97938660286659\
5+46.059959815245634%2C6.916114426884698+46.08272062168243%2C6.918457840809953+46.14607623301808%2C\
6.824721283799734+46.1509467247584%2C6.892680287632143+46.28714553466179%2C6.820034455949223+46.345\
413039583434%2C6.8481554230522885+46.44400246850344%2C6.616157444451996+46.47628810149786%2C6.46149\
2125385134+46.464990307615324%2C6.400563363328491+46.43754304454981%2C6.255271699962651+46.39877041\
861561%2C6.20371659360703+46.329233844940774%2C6.254277591648711+46.228445983291294%2C6.15682775621\
1207+46.176287491516945%2C5.998309357232866+46.15559023979242%2C6.024295980016201+46.21675959239505\
%2C6.139936451402039+46.236535104389105%2C6.14383444481954+46.31466847674628%2C6.198406352664541+46\
.3810390757354%2C6.097058523809538+46.431211909195355%2C6.110163898726027+46.47417421001637%2C6.203\
7157407460315+46.53856349406499%2C6.143946508344361+46.57787495635853%2C6.364832802002705+46.709895\
42612298%2C6.4921672536410435+46.76866555606827%2C6.481772604527709+46.951687666569256%2C6.58052177\
110438+46.96723299152839%2C6.658481639454383+46.983190940703594%2C6.770224117422721+47.077068943997\
49%2C6.848183985772725+47.13897333347006%2C6.954729139184396+47.21669385850086%2C6.9937090733593985\
+47.243163491709%2C6.978117099689397+47.28372458882927%2C7.076866266266068+47.32953884391242%2C7.07\
9464928544401+47.377073159692145%2C7.014498371586065+47.394667596712935%2C6.939137165514396+47.3841\
1163922788%2C7.017097033864398+47.443900779281364%2C7.0877888553547095+47.47346476569885%2C7.158201\
818637661+47.40798178903711%2C7.426651241153905+47.410960056197524%2C7.598282839156095+47.512120944\
3158%2C7.6355642067009555+47.606930768144366%2C7.56061264293914+47.684616570001396%2C7.599049342304\
173+47.89636842115823%2C7.65093888644697+48.012207141287774%2C7.593283837399419+48.0623232832113%2C\
7.73289985539644+48.29036520382502%2C7.777102059666228+48.315932884896824%2C7.790554904443988+48.47\
671484930484%2C7.857819128332798+48.51619444523979%2C7.852053623428044+48.634448805817215%2C7.99113\
9901027917+48.73849420592762%2C8.124578890569039+48.789981520359845%2C8.251508661108156+48.98045298\
840532%2C8.196180299591104+49.00821395264893%2C8.095287405060013+49.01034877067508%2C7.942320758512\
871+49.0828781279767%2C7.851191692484787+49.06155694573599%2C7.78935411196573+49.0892726983611%2C7.\
685206607933634+49.06795426121266%2C7.561531446895519+49.11484274545762%2C7.4606385523644265+49.212\
739233359656%2C7.346727219829322+49.20210768502247%2C7.301162686815279+49.15317311569822%2C7.177487\
525777166+49.14252876356113%2C7.105242492437857+49.16713544247557%2C7.061285290721069+49.1689317613\
3718%2C7.028317389433477+49.21381855394861%2C6.932161010678003+49.24252472956107%2C6.80853138084953\
8+49.23534974925877%2C6.825015331493332+49.17521836413715%2C6.745342903381656+49.18060624542767%2C6\
.736433034715952+49.22999562255367%2C6.701137085681321+49.23644895161709%2C6.687018706067468+49.288\
04523714476%2C6.600896590422968+49.34234725798254%2C6.624897835766516+49.373612761141516%2C6.551482\
261774484+49.44434646362572%2C6.449138155285681+49.48140718195751%2C6.3898409609075015+49.478655313\
41645%2C6.316425386915468+49.49057896025738%2C6.257128192537288+49.528165289030426%2C6.137121965819\
543+49.51166753013305%2C6.087707637171059+49.47865531341645%2C6.022763090947338+49.47039877992512%2\
C5.997350007642404+49.476820648476405%2C5.991702655796862+49.49699818276633%2C5.939464651225609+49.\
51441754313953%2C5.873108267040503+49.51808398664011%2C5.870284591117732+49.55106961006271%2C5.7516\
90202361372+49.57121653270477%2C5.722605803029234+49.55592947793582%2C5.676015150303521+49.56142437\
0801745%2C5.654837580882742+49.573327850964404%2C5.591304872620405+49.53027846885814%2C5.5404787060\
10537+49.53852490465928%2C5.482593349593742+49.51928105672678%2C5.4698868079412755+49.5797362150466\
1%2C5.410589613563095+49.63188728760085%2C5.342821391416603+49.64468778991434%2C5.351292419184915+4\
9.666623680339455%2C5.272229493347341+49.71046580230154%2C5.1945784054711535+49.71046580230154%2C5.\
159282456436522+49.73784705058449%2C5.092926072251416+49.7779782832404%2C5.04766944595205+49.785422\
02062795%2C5.001078793226337+49.81184993253504%2C4.933310571079845+49.81184993253504%2C4.9191921914\
65993+49.80182725011943%2C4.881072566508592+49.8054721020134%2C4.890955432238289+49.82733545099754%\
2C4.883514139590611+49.86840921352993%2C4.914380139300028+49.90818189536947%2C4.898947139445319+49.\
94171460660945%2C4.808278265298909+49.96653861432466%2C4.870010264717742+50.07685031035422%2C4.8977\
94021182952+50.08410264484593%2C4.9132270210376605+50.13977340184405%2C4.88621927129192+50.16820235\
39681%2C4.814841646963894+50.17438032439068%2C4.739605772672192+50.136064031468976%2C4.704881522999\
099+50.11009038558373%2C4.641220398598427+50.09276679132873%2C4.672086398307844+50.0481916363303%2C\
4.66436989838049+50.01473307725635%2C4.5582680243793705+49.989933929036%2C4.510039899833407+49.9626\
4008884654%2C4.452166150378251+49.952711217343875%2C4.293977901867492+49.98001068753646%2C4.2168129\
0259395+49.975048299107556%2C4.1570100281569555+50.03208486962538%2C4.253466277248882+50.0593393094\
0998%2C4.212954652630273+50.148427480968024%2C4.170513903029826+50.15460800745021%2C4.2071672776847\
57+50.22994612035674%2C4.2496080272852055+50.25585469158287%2C4.199450777757404+50.30270107462968%2\
C4.137718778338571+50.30146886568812%2C4.021971279428258+50.372884225758895%2C3.902365530554268+50.\
34580827878048%2C3.842562656117274+50.37165383593975%2C3.730673407170639+50.35811744199856%2C3.7094\
530323704156+50.32733855025549%2C3.686303532588353+50.372884225758895%2C3.6920909075338697+50.40486\
3162490585%2C3.6708705327336455+50.47612308867156%2C3.6207132832058444+50.509259221897565%2C3.54161\
9158950464+50.515392992491286%2C3.530044409059433+50.53869405321197%2C3.4567376597495687+50.5509531\
4792126%2C3.4162260351309595+50.51171282576638%2C3.387289160403382+50.51171282576638%2C3.3024076612\
02486+50.53992010609949%2C3.2701165930954867+50.65271284116184%2C3.2845399941501765+50.682843487179\
21%2C3.2710640810627587+50.72077706710368%2C3.2201550760658466+50.72930789395406%2C3.16026212901065\
63+50.803176675976516%2C3.101866505631845+50.809800223173056%2C3.015021732401819+50.79181983881577%\
2C2.907214427702476+50.752049172513466%2C2.9042197803497163+50.71224468698546%2C2.842829509618146+5\
0.7416274574054%2C2.81587768344331+50.73025566772745%2C2.747000794329841+50.81642283133385%2C2.6646\
47992128954+50.83533939505631%2C2.6451827843360167+50.83155669548043%2C2.6196705725285674+50.919256\
21255098%2C2.657103664438061+50.93813108772295%2C2.6091893067939087+51.006959398469945%2C2.58672945\
16482118+51.037098560518245%2C2.5403124176804392+51.11048070006885%2C2.3918625189210445+51.06992547\
139913%2C2.3245242285269385+51.07189349748759%2C2.221167782805753+51.04728715586913%2C2.13973543163\
14847+51.0531938705482%2C1.7873839121274413+50.978319812753995%2C1.6296337927162121+50.886844317193\
04%2C1.5601919390586991+50.873486405679984%2C1.4922466878166238+50.672082110060074%2C1.475181655989\
7704+50.25567091372406%2C1.1793877709909784+50.02233941106678%2C0.4399030584939979+49.8502614115587\
05%2C0.08153739013007666+49.740102956538095%2C-0.02085280083104372+49.49687180023221%2C0.1088744718\
8353104+49.44867583968477%2C-0.06746419032728733+49.348723896187124%2C-0.3860114510952177+49.389469\
61500665%2C-0.8012605588819838+49.41908165590635%2C-1.0629243802270687+49.47085976051329%2C-1.25063\
97303224561+49.758337379917606%2C-1.90479928368517+49.83177656369108%2C-2.0094884167208864+49.70236\
3868392524%2C-1.8861178730736312+49.603698611638066%2C-1.9272413876227164+49.5393804265608%2C-1.842\
5753282569532+49.36797586452755%2C-1.660298595090128+49.17196398667846%2C-1.6651366556253147+48.950\
05304462873%2C-1.9215538639901981+48.90873189080773%2C-2.1731330118198953+48.685645332624006%2C-2.3\
860076753681008+48.76224392267386%2C-3.1020406345756997+48.92462868920205%2C-3.3826481456165154+48.\
867376537342835%2C-3.6051989302350935+48.87374112569748%2C-3.701960140938823+48.82917200436799%2C-3\
.6390653539813993+48.74310521198882%2C-3.8712922596703505+48.75586516249405%2C-3.9051586834166554+4\
8.82917200436799%2C-4.161575891781539+48.75905464383757%2C-4.2692335295147+48.70457582362718%2C-4.3\
36966377007311+48.72372920411797%2C-4.680468675005551+48.608699527442724%2C-4.767553764638908+48.66\
624718578822%2C-5.3543515022541746+48.53918482054581))""",
        "Germany BB": """POLYGON((13.879508018493707+53.50106811523443%2C13.917368888855037+53.4205\
398559572%2C14.24040889739996+53.42845153808605%2C14.235617637634332+53.36948013305664%2C14.1000900\
26855584+53.261421203613224%2C14.258330345153865+53.25825119018583%2C14.319879531860293+53.31262207\
031256%2C14.420248985290582+53.32563018798834%2C14.448929786682186+53.26163864135748%2C14.367140769\
958668+53.18033218383795%2C14.388121604919602+53.13629150390619%2C14.366089820861816+53.07647705078\
131%2C14.142452239990462+52.961112976074446%2C14.15589046478277+52.87955856323245%2C14.121270179748\
82+52.84027099609369%2C14.341383934021167+52.75926971435547%2C14.466204643249453+52.65725708007818%\
2C14.639061927795465+52.58003234863307%2C14.600604057312069+52.53302383422857%2C14.631249427795467+\
52.499130249023494%2C14.528908729553168+52.39641189575201%2C14.584350585937784+52.31105804443365%2C\
14.570899963378904+52.2895622253418%2C14.699570655823036+52.24108886718773%2C14.669348716735897+52.\
12155151367199%2C14.741278648376522+52.07339096069347%2C14.680540084838867+51.905200958251896%2C14.\
58503341674816+51.837066650390625%2C14.738728523254451+51.66687011718744%2C14.739829063415698+51.60\
350036621108%2C14.685029983520735+51.5994300842288%2C14.665378570556811+51.55078887939459%2C14.5700\
5023956299+51.57242202758792%2C14.341999053955135+51.500820159912394%2C14.15363979339628+51.5450515\
74707%2C14.091679573059254+51.49647903442383%2C14.107279777526855+51.48025131225583%2C14.0498104095\
45955+51.48036193847656%2C14.080458641052418+51.44773864746094%2C14.015210151672363+51.374301910400\
62%2C13.97159862518322+51.39891052246094%2C13.779280662536564+51.36178207397472%2C13.56014919281022\
7+51.369880676269815%2C13.421508789062672+51.45396041870117%2C13.39611911773693+51.42473983764643%2\
C13.337078094482365+51.44050979614269%2C13.286639213562069+51.38579177856454%2C13.233359336853141+5\
1.39775848388672%2C13.222880363464583+51.431190490722685%2C13.19277858734148+51.427951812744254%2C1\
3.226088523865029+51.505062103271484%2C13.20432090759306+51.55712890625006%2C13.063730239868335+51.\
64789962768552%2C13.198930740356388+51.71620941162115%2C13.137908935546875+51.84654998779291%2C13.1\
63080215454045+51.87210083007807%2C13.048269271850645+51.87001037597648%2C13.055450439453068+51.899\
50942993167%2C12.982969284057617+51.900531768799084%2C12.97213935852062+51.93397140502944%2C12.8631\
8016052263+51.931758880615405%2C12.858329772949446+51.96512985229492%2C12.797579765319881+51.958541\
87011719%2C12.68417930603033+52.00812149047857%2C12.549958229064998+51.980232238769815%2C12.4788084\
03015138+52.03289031982416%2C12.435828208923567+52.01491165161133%2C12.221308708190918+52.165298461\
91406%2C12.301870346069334+52.21995162963867%2C12.25396919250494+52.25008010864258%2C12.31740856170\
6713+52.341960906982706%2C12.293430328369139+52.357040405273494%2C12.312949180603027+52.40127944946\
3146%2C12.28273010253912+52.41271972656256%2C12.33942985534668+52.47510147094738%2C12.2428398132323\
63+52.5206108093264%2C12.205368995666449+52.49135208129883%2C12.175079345703123+52.50278091430664%2\
C12.194099426269473+52.52854156494146%2C12.151088714599554+52.521560668945256%2C12.189039230346962+\
52.56937026977536%2C12.178129196167049+52.62139892578131%2C12.245609283447322+52.62440872192394%2C1\
2.24106884002697+52.68381118774414%2C12.204960823059082+52.710159301757926%2C12.218799591064396+52.\
769401550293026%2C12.26237869262718+52.79122161865246%2C12.239520072937294+52.854591369628906%2C11.\
846659660339297+52.90298080444353%2C11.82853889465349+52.921730041503906%2C11.853730201721191+52.94\
752883911144%2C11.693689346313647+52.974960327148466%2C11.688010215759562+53.00477981567377%2C11.63\
239002227806+53.00897216796881%2C11.639038085937614+53.03868865966814%2C11.521370887756461+53.04336\
929321289%2C11.459699630737418+53.07736206054685%2C11.34766864776617+53.05216217041044%2C11.2733802\
79541016+53.101150512695455%2C11.39784812927252+53.10763168334961%2C11.39811038970953+53.1374320983\
8873%2C11.540950775146655+53.121391296386804%2C11.578739166259766+53.16205215454124%2C11.5669193267\
82282+53.21057128906253%2C11.635769844055346+53.239810943603516%2C11.834399223327923+53.22312164306\
6406%2C11.803829193115234+53.24948883056646%2C12.028409957885799+53.29949951171881%2C12.05480003356\
945+53.366352081298885%2C12.171999931335733+53.33528137207048%2C12.234510421753102+53.3532485961914\
6%2C12.264590263366701+53.319358825683736%2C12.400289535522461+53.29916000366214%2C12.4481792449951\
19+53.24644088745109%2C12.6708602905274+53.25107192993181%2C12.676119804382608+53.22494888305667%2C\
12.762390136718976+53.22008132934582%2C12.742948532104776+53.194301605224666%2C12.767208099365234+5\
3.182811737060604%2C12.977380752563704+53.19474029541021%2C12.945539474487248+53.169181823730526%2C\
12.982050895691202+53.157501220703125%2C13.121039390563908+53.237201690674084%2C13.1831893920899+53\
.24739074707037%2C13.231020927429368+53.21321105957048%2C13.301878929138354+53.275329589843864%2C13\
.405271530151367+53.243896484375%2C13.438720703125227+53.29528808593773%2C13.526306152343807+53.316\
10107421889%2C13.54931640625+53.39727783203125%2C13.624085426330737+53.407287597656364%2C13.7094182\
9681425+53.47985076904297%2C13.829074859619368+53.49626159667983%2C13.78027439117426+53.51208496093\
75%2C13.789619445800952+53.556289672851506%2C13.818076133728255+53.51507186889651%2C13.879508018493\
707+53.50106811523443)%2C(13.482250213623216+52.67502212524411%2C13.45722961425804+52.6605491638183\
3%2C13.474988937378214+52.649169921875%2C13.370199203491438+52.62105941772461%2C13.28633975982666+5\
2.655658721924084%2C13.272738456726358+52.62248992919922%2C13.230238914489973+52.62681198120117%2C1\
3.21620941162138+52.58251953125006%2C13.13699054718029+52.583648681640824%2C13.160609245300407+52.5\
6478118896493%2C13.122269630432298+52.52083206176769%2C13.176479339599553+52.505241394043%2C13.0994\
59648132324+52.40625%2C13.141288757324162+52.38713073730469%2C13.30657005310087+52.41442108154297%2\
C13.432428359985524+52.375541687011776%2C13.427828788757438+52.40896224975589%2C13.476848602295206+\
52.419349670410156%2C13.487819671630858+52.39324188232442%2C13.650609970092829+52.37594223022461%2C\
13.660959243774416+52.33872222900396%2C13.74246978759777+52.40044021606468%2C13.762909889221136+52.\
448318481445256%2C13.618619918823356+52.4690895080567%2C13.663810729980526+52.52770996093767%2C13.5\
14860153198356+52.5892715454101%2C13.529389381408748+52.64094924926766%2C13.482250213623216+52.6750\
2212524411))""",
        "Germany LS": """POLYGON((6.538349462786643+53.622403757511705%2C6.538349462786643+53.62240\
757511705%2C6.587787939349144+53.52128430954264%2C7.076679540911643+53.26906926552297%2C7.142597509\
661643+53.292059692314695%2C7.159077001849144+53.180276012687784%2C7.148090673724144+53.09459731629\
5165%2C7.170063329974144+53.01205274308296%2C7.010761572161643+52.816620149547504%2C7.0052684080991\
45+52.696939623704395%2C6.681171728411644+52.676960891019604%2C6.659199072161644+52.46663195499079%\
2C7.010761572161643+52.36612050895073%2C6.988788915911643+52.251930446466645%2C7.054706884661644+52\
.19809199182603%2C7.274433447161644+52.235112941544486%2C7.565571142474145+52.335922390499036%2C7.6\
36982275286643+52.42980435752739%2C7.757831884661643+52.36276618123409%2C7.856708837786644+52.28218\
589518116%2C7.851215673724144+52.19809199182603%2C7.911640478411644+52.12395749657327%2C7.845722509\
661644+52.08009269084269%2C7.972065283099144+51.99899813474639%2C8.301655126849143+52.0800926908426\
9%2C8.389545751849145+52.06996391619205%2C8.537861181536643+52.15092978318316%2C8.576313329974145+5\
2.21492352147874%2C8.521381689349143+52.248567455379714%2C8.510395361224145+52.32249438164985%2C8.4\
77436376849143+52.37618196412171%2C8.384052587786643+52.416404868890766%2C8.570820165911645+52.4666\
3195499079%2C8.636738134661645+52.39294596328108%2C8.878437353411645+52.29226645693046%2C8.93336899\
4036643+52.211557725615535%2C9.032245947161645+52.08684393064243%2C9.120136572161643+52.07334042974\
5855%2C9.125629736224143+51.99223361336334%2C9.258347089975826+51.85825711495349%2C9.37919669935082\
6+51.817527161148035%2C9.324265058725826+51.71554102944424%2C9.357224043100826+51.6099131933598%2C9\
.549484785288326+51.57920091442452%2C9.549484785288326+51.49719995087838%2C9.533005293100825+51.432\
17771250704%2C9.505539472788326+51.34304972442618%2C9.747238691538326+51.26062360014694%2C9.8241429\
88413326+51.31559079578369%2C9.988937910288325+51.36020320545316%2C10.324020918100828+51.4527210605\
71804%2C10.428391035288325+51.54163547882871%2C10.692062910288326+51.538218900432156%2C10.757980879\
038326+51.654238717782505%2C10.659103925913328+51.75976330992552%2C10.642624433725826+51.8378967435\
2599%2C10.736008222788325+51.90572883688944%2C10.675583418100828+51.99375765169381%2C11.01615958997\
5826+52.02756888786209%2C11.115036543100828+52.18277168218276%2C11.147995527475828+52.2467196225889\
3%2C11.076584394663326+52.47148611636476%2C11.038132246225826+52.54504084650378%2C11.00517326185082\
6+52.648477305483766%2C10.840378339975828+52.808155495071645%2C11.499558027475826+52.86788412632006\
4%2C11.647873457163326+53.033366265465276%2C11.631393964975826+53.08618689494182%2C11.3896947462258\
28+53.12246351923477%2C11.290817793100828+53.191633957606655%2C11.06010490247583+53.336201323544174\
%2C10.977707441538328+53.388650153670596%2C10.510788496225828+53.49335422415729%2C9.923019941538326\
+53.52602134692671%2C9.730759199350828+53.59780041373591%2C9.554977949350828+53.67271198732601%2C9.\
417648847788328+53.796187671749266%2C9.137497480600828+53.89664935819215%2C8.665085371225828+53.922\
53583010114%2C7.813644941538328+53.82862101043645%2C6.577683027475827+53.688979513747796%2C6.538349\
462786643+53.622403757511705))""",
        "Germany NRW": """POLYGON((5.945237203125001+51.92270006676995%2C5.8738260703125+51.8005747\
4206038%2C6.1484842734375+51.490389338595804%2C6.1265116171875+51.37737980904703%2C5.97819618750000\
1+51.188408948720564%2C6.0276346640625+51.14707558564015%2C5.923264546875+51.09190680068855%2C5.840\
8670859375+51.047033645419134%2C5.8903055625+50.92946732137099%2C5.961716695312499+50.8809710843845\
%2C5.895798726562499+50.78729992164125%2C6.1484842734375+50.599393290260224%2C6.1814432578125+50.48\
41907174694%2C6.291306539062501+50.37221027865061%2C6.483567281249999+50.29507006596933%2C6.7856913\
04687499+50.31962826517807%2C6.868088765624998+50.35468940954428%2C6.868088765624998+50.42123430628\
1434%2C7.087815328124999+50.5575340045454%2C7.439377828125+50.641215379146644%2C7.714036031249999+5\
0.742131926951004%2C7.829392476562498+50.82895475933529%2C8.087571187499998+50.641215379146644%2C8.\
235886617187498+50.71431456232472%2C8.279831929687498+50.82895475933529%2C8.455613179687498+50.8393\
6266572633%2C8.5764627890625+50.96061680013034%2C8.81266884375+51.1332895653552%2C8.779709859375+51\
.288145023879736%2C9.032395406250002+51.36366268401932%2C9.372971578125+51.48012729562444%2C9.51030\
06796875+51.69514548666476%2C9.515793843750002+51.84132002209577%2C9.2905741171875+52.0748907977108\
8%2C9.109299703125002+52.18953639697236%2C9.098313375000005+52.30724592442749%2C9.175217671875004+5\
2.42799294356129%2C9.186204000000004+52.504964155438245%2C9.048874898437504+52.51833671603367%2C8.9\
06052632812505+52.42799294356129%2C8.774216695312504+52.431342317049484%2C8.735764546875004+52.5550\
90278715824%2C8.543503804687505+52.54172890648684%2C8.252366109375004+52.474861014383606%2C8.235886\
617187504+52.40118879215959%2C8.411667867187504+52.32067856076711%2C8.406174703125005+52.2366576856\
4655%2C8.186448140625004+52.12550610621196%2C8.060105367187502+52.09851842285795%2C8.07658485937500\
4+52.18280070357636%2C8.049119039062504+52.27364649610661%2C8.010666890625004+52.37772179404482%2C7\
.845871968750004+52.424643315550014%2C7.768967671875004+52.42799294356129%2C7.719529195312504+52.48\
4897684160615%2C7.582200093750004+52.49827634912731%2C7.4943094687500045+52.41459290429816%2C7.2965\
55562500004+52.31396275217705%2C6.890061421875004+52.23665768564655%2C6.670334859375004+52.10189277\
668701%2C6.6428690390625045+52.02421803863098%2C6.7362528281250045+51.96333485703687%2C6.3242655234\
37504+51.88880961792111%2C6.093552632812504+51.95317961036827%2C5.945237203125001+51.92270006676995\
))""",
        "Latvia": """POLYGON((21.611733183152086+57.569516560536385%2C21.330521218583826+57.2571456\
56914446%2C21.356884840262097+57.06175290788224%2C20.96143051508798+56.83168639885798%2C20.87355177\
6160394+56.44993642532111%2C20.987794136766254+56.02504451360812%2C21.198703110192454+56.0250445136\
0812%2C21.602945309259326+56.28444767218994%2C22.156581364503094+56.38188101103927%2C22.95627788874\
4093+56.32832345839887%2C23.202338357741322+56.211209328441214%2C23.369307961703726+56.347807635915\
19%2C23.99324700808956+56.279569473821596%2C24.20415598151576+56.17209157379639%2C24.81930715400883\
6+56.38188101103927%2C25.03900400132779+56.13293392606741%2C25.619003678249832+56.079026981420725%2\
C26.427488076383586+55.65992240052077%2C26.893245392699775+55.63512818869191%2C27.139305861697+55.8\
0341948634242%2C27.63142679969146+55.74905164449527%2C28.149911359364197+56.12313827821754%2C28.352\
032458897636+56.29907853355192%2C28.03566899875834+56.9037347195119%2C27.824760025332143+56.9325151\
9440378%2C27.991729629294547+57.29515078815578%2C27.64900254747698+57.44677935364845%2C27.622638925\
7987+57.58836258133488%2C26.84051814934322+57.687144160166554%2C26.620821302024265+57.6166133162688\
25%2C26.11112461624429+57.88390216370402%2C25.355367461467086+58.12602439999736%2C25.03900400132779\
+58.1353041117259%2C24.335974089907133+57.97255981678262%2C24.26101401326246+57.89630821130572%2C24\
.321998491905013+57.2477340495476%2C23.613900934333124+56.999454256227324%2C23.535976322734303+57.0\
6398297523779%2C23.298814461346588+57.09896594596549%2C23.187009583835234+57.353891831577016%2C22.6\
48313355826005+57.6233912248629%2C22.67880559514728+57.7772735323928%2C22.573776770818437+57.782692\
73379826%2C21.611733183152086+57.569516560536385))""",
        "Lithuania": """POLYGON((21.043175076113204+56.07733684517308%2C21.024925450314083+55.91275\
945357236%2C21.07739312448656+55.67294001831317%2C21.081955530936334+55.54021556414412%2C21.0408938\
72888322+55.41869607866778%2C20.92683371164383+55.27991427094318%2C21.013519434189643+55.2630197819\
4766%2C21.100205156735456+55.37076111572759%2C21.154954034132807+55.607278221178035%2C21.2302337405\
5417+55.48208809647244%2C21.223390130879505+55.4070415719477%2C21.164078847032368+55.3331498279378%\
2C21.25532697602796+55.318874130494635%2C21.27357660182708+55.30069762369939%2C21.24392095990351+55\
.2747167320704%2C21.24392095990351+55.242216697737774%2C21.298669837300867+55.23831490712081%2C21.3\
83074356621787+55.286410237839874%2C21.45607285981826+55.20057785492648%2C21.515384143665397+55.173\
22867894106%2C21.56785181783786+55.18885907834792%2C21.643131524259225+55.16801718333605%2C21.70187\
5964403172+55.14515845369735%2C21.708887813642786+55.12912616681015%2C21.789524079898353+55.1080740\
1652082%2C21.814065552237004+55.11409004884908%2C21.849124798435078+55.08299746333333%2C21.97007919\
781843+55.06493260375359%2C22.010397330946212+55.08199407400309%2C22.01916214249573+55.040833434347\
32%2C22.078762861032452+55.01269940126741%2C22.140116541879085+55.02073971169517%2C22.1593991272880\
22+55.04685958654761%2C22.35047201906752+55.05388895159561%2C22.43286124763299+55.04384662377839%2C\
22.4854501169301+55.036815495839136%2C22.57309823242528+55.065936421057984%2C22.588874893214417+55.\
00666811028063%2C22.64321672482143+54.95637207024299%2C22.692299669498727+54.96241092232597%2C22.74\
1382614176032+54.93422184658232%2C22.743258027296008+54.917281992089784%2C22.764795647840756+54.911\
71108830946%2C22.775564458113138+54.917281992089784%2C22.787410149412747+54.900566967246846%2C22.80\
4640245848546+54.895613034321656%2C22.821870342284345+54.9055202907835%2C22.841254200774618+54.8869\
4218524725%2C22.828331628447767+54.860298619949646%2C22.85309989207423+54.85286006288342%2C22.84663\
8605910805+54.83487788569022%2C22.86925310748279+54.82867527676632%2C22.858484297210417+54.79702714\
535438%2C22.83586979563843+54.78212532701713%2C22.797102078657883+54.76224769165901%2C22.7637187668\
1352+54.754791060460576%2C22.749719313459433+54.739873677081505%2C22.728181692914685+54.73179113325\
912%2C22.728181692914685+54.70815598940709%2C22.71310535853336+54.683884536972556%2C22.728181692914\
68+54.67443228284927%2C22.718489763669545+54.66259971443844%2C22.743258027296005+54.651386732696096\
%2C22.734642979078107+54.63206823202083%2C22.706644072369933+54.61710566829612%2C22.706644072369933\
+54.60962232258393%2C22.666799474362147+54.587788037305415%2C22.699105905179273+54.56344461200064%2\
C22.666799474362147+54.5278395985155%2C22.687260213879657+54.50721196575654%2C22.695314682229366+54\
.46009051542836%2C22.6921119470868+54.446436112763024%2C22.729477190416723+54.43898633714426%2C22.7\
583018066998+54.38120461785678%2C22.78926157974459+54.35819354607949%2C22.822356509551092+54.365036\
02242922%2C22.839437763644767+54.39985277893146%2C22.880005742117252+54.404202797157865%2C22.882140\
89887896+54.395502299352586%2C22.995304207249575+54.376852160783045%2C22.984628423441023+54.3557050\
90266014%2C23.048683126292314+54.34201588897167%2C23.029466715436925+54.31462380518093%2C23.0967241\
53430785+54.29344469793301%2C23.139427288664972+54.30839578709393%2C23.152238229235234+54.306527197\
872896%2C23.14049486704583+54.29344469793301%2C23.197076521231132+54.28222781763418%2C23.2301714510\
37634+54.255419570772204%2C23.248320283512168+54.25043006611952%2C23.27928005655696+54.253548577264\
81%2C23.337996867503975+54.24543995782227%2C23.35401054321679+54.22672167489799%2C23.37856484597645\
2+54.22172869880899%2C23.41379493254467+54.17176572056928%2C23.449025019112874+54.169265985931276%2\
C23.475714478634245+54.14863741112623%2C23.486390262442793+54.10547170338785%2C23.507741830059892+5\
4.086689975708225%2C23.516282457106726+54.064140675933004%2C23.514147300345016+54.03593682408359%2C\
23.493863311108775+54.013359957055854%2C23.47037658672997+53.99767436583724%2C23.480128391710537+53\
.97207102949835%2C23.4986342377817+53.95483326200835%2C23.518682237692126+53.947573121901655%2C23.5\
3410377608476+53.93395695224973%2C23.568031160548557+53.93304904958978%2C23.59116346813751+53.93032\
522308991%2C23.60041639117309+53.917611681965724%2C23.71453577527859+53.92850923998796%2C23.7685111\
59652814+53.91034506277515%2C23.76234254429576+53.893081774018356%2C23.813233620991454+53.894899298\
23978%2C23.84253454393746+53.91307019315292%2C23.808607159473667+53.932141127176635%2C23.8487031592\
94516+53.95029582256089%2C23.88417269759758+53.93304904958978%2C23.922726543579167+53.9512033499483\
15%2C23.9551117742037+53.94031171779656%2C23.952027466525173+53.92487703673183%2C23.979786235631916\
+53.913978530429944%2C24.19106131161102+53.9493882754241%2C24.238868080628187+53.93849616925843%2C2\
4.34219238785884+53.89035533947023%2C24.46093823348213+53.896716743414544%2C24.625948694283327+54.0\
00180506665004%2C24.68145266011555+53.997384995490506%2C24.663791908158306+53.951681853159926%2C24.\
762692119118856+53.959995244326194%2C24.911042435559686+54.040963969630866%2C24.80507792381624+54.1\
05211423838995%2C24.953428240257068+54.15488312646582%2C25.09118210552355+54.111423643744814%2C25.2\
28935970790037+54.22721431394524%2C25.274853925878865+54.25404795591791%2C25.370221986447966+54.245\
79330932835%2C25.46559004701707+54.28292621137436%2C25.546829506020384+54.24166536652413%2C25.48325\
079897431+54.16522390294739%2C25.67045476972107+54.111423643744814%2C25.82233723655335+54.159019747\
12801%2C25.808208634987555+54.24372938955409%2C25.691647672069756+54.34268111218171%2C25.5856831603\
26308+54.33856286990661%2C25.659858318546725+54.42084935505528%2C25.656326168155278+54.461930763383\
975%2C25.77641928146452+54.57469228093018%2C25.771179187588896+54.66683503329301%2C25.7755503831824\
+54.805636822585626%2C25.906686250987523+54.91633611544577%2C26.11650363947572+54.938941729372885%2\
C26.26075309406136+55.01169565774333%2C26.321949832370418+55.109329433412285%2C26.57547917679366+55\
.12932744874928%2C26.754698196127332+55.16679674567018%2C26.88146286833895+55.31632231100792%2C26.5\
3176722085862+55.36355490715832%2C26.592963959167676+55.39583950671229%2C26.57547917679366+55.50243\
9405466305%2C26.68038787103776+55.59146446256696%2C26.67164547985075+55.687680897766995%2C26.802781\
347655873+55.66303254393122%2C26.933917215461+55.71970054518195%2C26.54925200323264+55.702462456617\
01%2C26.326321027963928+55.766451312198114%2C26.225783529313333+55.881857453859965%2C26.01596614082\
513+55.99447690143833%2C25.744952014027874+56.106769187054766%2C25.6706416889383+56.179827274776784\
%2C25.119871044156774+56.211442670256105%2C24.910053655668577+56.470673229560305%2C24.6171835509037\
96+56.4005884296908%2C24.534130834627216+56.32552723894068%2C24.45107811835064+56.28187547027409%2C\
24.31557105495201+56.32067950510958%2C24.14946562239885+56.27702219474526%2C24.000844972219706+56.3\
6428695426798%2C23.80851236610552+56.35944414343561%2C23.799769974918515+56.39575023163832%2C23.528\
755848121257+56.36428695426798%2C23.345165633194085+56.40784457393582%2C23.161575418266906+56.38849\
178161669%2C23.11786346233187+56.34975667605974%2C23.052295528429305+56.35460071738069%2C23.0304395\
50461786+56.43443860264955%2C22.912417269437174+56.44410454906722%2C22.86870531350213+56.4030072981\
4484%2C22.69385748976195+56.38849178161669%2C22.383502602623153+56.432021731967495%2C22.24362434363\
102+56.432021731967495%2C22.147458040573934+56.44652065162131%2C22.07314771548436+56.44652065162131\
%2C21.928898260898723+56.415099334911304%2C21.898299891744195+56.38607199080292%2C21.80213358868710\
2+56.38607199080292%2C21.710338481223513+56.34006674744546%2C21.561717831044373+56.34248946035916%2\
C21.408725985271726+56.26488631099869%2C21.212022183564038+56.182260153362506%2C21.1726814232225+56\
.10920669417163%2C21.043175076113204+56.07733684517308))""",
        "Netherlands": """POLYGON((6.435415229305765+53.57977751133939%2C6.3399385978089455+53.5293\
4619640311%2C5.918177970273683+53.47917668086342%2C5.717030594064559+53.46244031223674%2C5.63916709\
359651+53.486898932862886%2C5.176311840814222+53.40574515921625%2C5.050865090060143+53.324436282038\
65%2C4.815111713642997+53.22873234778379%2C4.691827837901921+53.03538053256804%2C4.629104462524882+\
52.96118351528588%2C4.693990712914922+52.936422837106505%2C4.5923355873038565+52.655247626645064%2C\
4.520960711874812+52.451411138226405%2C4.36307083592571+52.20555611078595%2C4.08838570927454+52.008\
95243567501%2C3.9283329583124384+51.98764604559051%2C3.965101833533461+51.89164176248059%2C3.850451\
729553719+51.83534196120448%2C3.819380626718985+51.76671986603955%2C3.655147654592537+51.7337441543\
1948%2C3.655147654592537+51.645691191153304%2C3.632954009710584+51.60987073802832%2C3.4864759534896\
98+51.59884336964819%2C3.3932626449854975+51.52157678496738%2C3.535301972229994+51.43034400169597%2\
C3.286733149552126+51.37496257465733%2C3.331120439316031+51.33060913809615%2C3.3355591682924217+51.\
28066018844291%2C3.4243337478202314+51.20841570727157%2C3.5574956171119463+51.23065686523174%2C3.59\
30054489230696+51.26677583148012%2C3.7527996920731272+51.244552132588154%2C3.783870794907861+51.172\
25090458939%2C3.9525424960107003+51.16946782107115%2C4.281008440263596+51.33060913809615%2C4.471873\
786248387+51.361106738067974%2C4.462996328295606+51.43311131277159%2C4.529577262941464+51.397123193\
217%2C5.031153637273589+51.38604422538492%2C5.275283730975067+51.22509758312165%2C5.50165890877098+\
51.23621547580308%2C5.781298834283581+51.11934359252598%2C5.625943320109915+50.881909962456064%2C5.\
563801114440448+50.80904041189078%2C5.705840441684943+50.730438189801845%2C6.012112741055887+50.719\
19851821417%2C6.074254946725354+50.75010112769567%2C6.047622572867011+50.82586656459435%2C6.1142035\
07512869+50.85669876708539%2C6.158590797276774+50.918302063890195%2C6.078693675701745+50.9658488249\
7925%2C6.012112741055887+51.060796517170814%2C6.2429266478281935+51.16111756272096%2C6.220733002946\
241+51.22787730812084%2C6.118642236489259+51.23065686523174%2C6.265120292710146+51.36942074313424%2\
C6.256242834757364+51.5160527192015%2C6.136397152394822+51.62365118250713%2C6.140835881371211+51.69\
799329523113%2C6.0209901990086685+51.80241639026971%2C6.2251717319226305+51.83534196120448%2C6.4559\
85638694936+51.80241639026971%2C6.682360816490851+51.879205301227955%2C6.806645227829785+51.8956430\
307571%2C6.9131747232631575+51.99414320202649%2C6.780012853971442+52.05696065690404%2C7.05965277948\
4044+52.20409853984532%2C7.096143766905282+52.299780325057895%2C7.09170503792889+52.43529324442753%\
2C7.007369187377472+52.50289388109792%2C6.874207318085757+52.492084755959894%2C6.767677822652384+52\
.492084755959894%2C6.749922906746821+52.5353053146674%2C6.8076263834399+52.62970213644476%2C7.10502\
1224858063+52.632396201908165%2C7.136092327692796+52.83934146311475%2C7.278131654937291+52.99457412\
264499%2C7.233744365173386+53.33253965617121%2C7.202829797844952+53.36262124463863%2C7.038596825718\
504+53.35467408917182%2C6.918751143355959+53.49221509474813%2C6.603601386032235+53.63983894257754%2\
C6.435415229305765+53.57977751133939))""",
        "Portugal": """POLYGON((-25.13577220842846+38.00076629773602%2C-25.78026708493708+37.985374\
87729621%2C-26.83489506467846+38.67478539142883%2C-27.04972669018133+38.91832611096666%2C-27.928583\
339965815+39.327411448852416%2C-28.4754274776095+39.02461333180199%2C-31.30729890469284+39.97400351\
0275196%2C-31.61978126906065+39.357619509028865%2C-28.319186295425588+38.18521130169799%2C-25.07718\
176510949+36.58697858099039%2C-16.616075995632038+32.31393915981381%2C-16.342653926810197+29.804758\
846765495%2C-15.4833274247987+29.906387816919377%2C-16.362184074583187+32.29743210708206%2C-16.0301\
71562442384+33.03726055924017%2C-8.818225306818176+36.99435184550987%2C-7.6133919019443566+36.94978\
8140267785%2C-7.278715956146075+37.11899134065078%2C-7.3233394155858464+37.59782244832468%2C-6.8970\
38143205281+38.002007125228715%2C-6.729700170306138+38.38777674041688%2C-7.064376116104423+38.37903\
195539727%2C-7.1089995755441935+38.69317760799811%2C-6.830102954045624+38.98860957562928%2C-7.25402\
5818723449+39.59298244496185%2C-6.941661602645052+39.601578524063086%2C-6.729700170306138+40.149505\
977200334%2C-6.731170207119339+41.00149442324107%2C-6.0060389912230585+41.529770883051555%2C-6.3630\
26666741226+41.821409514445975%2C-6.4522735856207705+42.020626183913805%2C-7.3670545041360755+42.08\
6893498545265%2C-7.947159476853101+41.987466600955116%2C-8.125653314612185+42.210958323272706%2C-8.\
951187314247951+42.020626183913805%2C-9.285863260046234+41.99575811686822%2C-8.940031449388007+41.1\
9485255027166%2C-8.884252125088294+40.82445315423834%2C-9.051590097987436+40.256441631619055%2C-9.2\
63551530326348+39.58911817618778%2C-9.575915746404744+39.39110303117326%2C-9.698630259864116+38.680\
55541513118%2C-8.940031449388007+38.26131629092316%2C-8.93978832897369+37.424288546428286%2C-9.0513\
6472358876+37.06011053569746%2C-15.977794185180707+33.20413116487465%2C-16.357153926871952+33.37201\
547156853%2C-16.94850881833183+33.0452775586776%2C-17.506390791407192+32.90487422691581%2C-24.85663\
504851792+36.792542943772375%2C-25.13577220842846+38.00076629773602))""",
        "Slovakia": """POLYGON((17.977334072618074+49.10580183648216%2C17.47534782656898+48.8822999\
623126%2C17.0719660217081+48.929436327125444%2C16.838900978899595+48.586689845339464%2C16.749260577\
819395+48.354906633979994%2C17.06300198160008+48.03222712750136%2C17.311669827284206+47.93756865892\
496%2C17.79000371387852+47.65030744376321%2C18.855383734020396+47.7478620528438%2C18.96419323172044\
4+47.80490566986916%2C18.841584340848954+47.918805189692904%2C19.001126220460513+48.01206932666699%\
2C19.435975208273522+48.04599680585349%2C19.638904735919592+48.18148341532353%2C19.936051544258483+\
48.07505972648616%2C20.276683251378675+48.21046978754151%2C20.450622846503876+48.292508798033765%2C\
20.57383005971756+48.48982626028891%2C20.8492344186658+48.53783642667179%2C21.16087619326513+48.451\
38536914391%2C21.458023001604012+48.513837030132834%2C21.702561630818813+48.353264717736806%2C21.78\
842481815809+48.26284092035701%2C22.203430223631262+48.36277369288331%2C22.267827614135715+48.58572\
3828159075%2C22.50395137931873+48.73695929002653%2C22.611280363492824+49.047450162694986%2C22.57550\
403543479+49.14115776588093%2C22.124722301903592+49.2487028839046%2C21.96730645844825+49.3979420481\
9239%2C21.5737668498099+49.47239206001166%2C21.41635100635456+49.45379015129686%2C21.24462463167600\
7+49.51886593875591%2C20.865395554260868+49.39794204819239%2C20.715134976417133+49.48169036621323%2\
C20.2285769148279+49.44913857065421%2C19.999608415256496+49.28605508163625%2C19.885124165470792+49.\
28138760331353%2C19.87796889985919+49.43983408513364%2C19.69193199395742+49.44448654860355%2C19.548\
826681725295+49.65802287776151%2C19.248305526037832+49.579216017683535%2C19.076579151359276+49.4723\
9206001166%2C19.047958088912853+49.57457635077958%2C18.47553683998434+49.5606547034144%2C18.2608788\
71636148+49.37465324127689%2C17.977334072618074+49.10580183648216))""",
        "Slovenia": """POLYGON((13.583661241564558+46.44808802135424%2C13.393517602376361+46.363499\
11335333%2C13.332180944573718+46.295733540934265%2C13.384317103705966+46.18755861560592%2C13.620463\
236246145+46.16207469437762%2C13.454854260179006+46.07704299687168%2C13.448720594398742+45.97909430\
809992%2C13.589794907344821+45.93645378296094%2C13.556059745553368+45.84039262387%2C13.574460742894\
16+45.761284546004504%2C13.86274303456659+45.63276241005167%2C13.654198398037598+45.60702253473349%\
2C13.4640547588494+45.51898880151799%2C13.654198398037598+45.42005503551053%2C13.76767121497249+45.\
42866486537511%2C13.816740541214607+45.40067811725055%2C13.97314901861135+45.42866486537511%2C14.04\
368617508439+45.46309105385876%2C14.285965973404835+45.45018369522185%2C14.522112105945016+45.52113\
7607003965%2C14.57118143218713+45.61131333454438%2C14.653985920220704+45.56409648331481%2C14.657052\
753110833+45.49104686227324%2C14.837995893628634+45.43942530615473%2C14.963736042124054+45.48889690\
787166%2C15.175347511543178+45.398524715983285%2C15.408426811193227+45.46094003258841%2C15.44216197\
298468+45.50824354311541%2C15.328689156049789+45.61345861136053%2C15.42989464142415+45.643483870616\
82%2C15.377758482291902+45.73560375005462%2C15.592436784601158+45.83398263969866%2C15.6660407739643\
31+45.79550722830544%2C15.748845261997902+45.83184581422171%2C15.767246259338693+46.074915523083064\
%2C15.668596834715302+46.113004780424035%2C15.686997832056097+46.19798100617649%2C15.76673548719953\
3+46.1937353112651%2C15.852606808123234+46.248903759703694%2C16.101020272223945+46.325199330990586%\
2C16.15622326424632+46.38023560308042%2C16.324899073203593+46.36753984406565%2C16.364767900775313+4\
6.3929284107104%2C16.300364410082537+46.48591882489936%2C16.4015698954569+46.50280912286638%2C16.57\
8311352989857+46.45413411689316%2C16.632115571920174+46.45678173902442%2C16.616742937940085+46.4925\
1203910552%2C16.53987976803963+46.55597465114093%2C16.42928457727765+46.642944620622586%2C16.469637\
74147539+46.69832459103358%2C16.340891931892124+46.78786710143129%2C16.371637199852305+46.844412819\
32311%2C16.287087712961803+46.893020845759594%2C16.14681242789347+46.87988788211351%2C15.9757918748\
64955+46.85098404392437%2C15.975791874864955+46.725993297865784%2C15.827830272806576+46.74179761888\
297%2C15.610691817837786+46.71150193480895%2C15.524220751699772+46.68382579767601%2C15.524220751699\
772+46.646902214657274%2C15.476181270511988+46.632389621049214%2C15.435828106314249+46.669323112086\
4%2C15.23214070607804+46.66009210376063%2C15.076492787029615+46.670641698888176%2C14.98425698314906\
8+46.642944620622586%2C14.917001709486168+46.648221348329514%2C14.845903277328246+46.60995341065941\
5%2C14.788255899902904+46.524082158508264%2C14.693369971290888+46.50991291234675%2C14.6510952278456\
37+46.45963518477811%2C14.558859423965087+46.44639648056187%2C14.562702582460114+46.405336045632%2C\
14.520427839014863+46.447720495800326%2C14.460858882342007+46.43977592164134%2C14.432035193629336+4\
6.460958878202774%2C14.170700415967785+46.44904447885705%2C14.134190410265065+46.4927178662827%2C14\
.005444600681802+46.50330007666287%2C13.934346168523879+46.5337124600116%2C13.805600358940614+46.52\
578043584009%2C13.688384024842419+46.52974659272198%2C13.67493297010984+46.462282539446505%2C13.583\
661241564558+46.44808802135424))""",
        "Spain NA": """POLYGON((-1.7439054951828712+43.39009402745177%2C-2.0864117783846448+43.1217\
0185467579%2C-2.329201042426408+42.909312180845774%2C-2.485279855024684+42.68344108228112%2C-2.6543\
652353394838+42.57498558484244%2C-2.3725562681481516+42.36071149986884%2C-1.98235923665246+42.23564\
7144788516%2C-1.9086553529254964+42.16498846093302%2C-2.0127078946576806+42.01699230783183%2C-1.704\
8857920333023+41.884791720356844%2C-1.3970636894089232+41.83635715777805%2C-1.18462308337238+42.020\
21328691927%2C-1.327695328254134+42.235647144788516%2C-1.1672809930836825+42.33828234233093%2C-0.94\
61693419027911+42.44075029511609%2C-1.0372153159184525+42.56859996508132%2C-0.7727484390158171+42.7\
21674371095816%2C-0.642682761850587+42.91883787435103%2C-0.7293932132940738+43.02986216202203%2C-1.\
1976296510889033+43.099546255264585%2C-1.3407018959706565+43.311277954245014%2C-1.7439054951828712+\
43.39009402745177))""",
        "Sweden": """POLYGON((14.233971893326936+55.367302917260474%2C14.440112662856382+55.5381557\
58714%2C14.277578594573548+55.72166806460086%2C14.305610875139992+55.89511512542242%2C14.5092667793\
23715+55.965273006347815%2C14.509266779323715+56.015681886492274%2C14.722286173354968+55.9685483059\
0249%2C14.899134248390308+55.99520966975305%2C14.797417811886799+56.113413191652825%2C15.3912218736\
3701+56.08274580185031%2C15.55204380702769+56.06203150317748%2C15.684000265194406+55.99136581921451\
black%2C15.689498450951353+55.93597197953554%2C15.799462166090281+55.93366218193469%2C15.8338258270\
71198+56.03823468804947%2C16.036729996720823+56.19726379239552%2C16.32753237886994+56.5929770833469\
66%2C16.316536007356046+56.11470971198378%2C16.56945255217558+56.145351648738426%2C17.1962457284674\
67+57.34816321452041%2C18.188399886319214+56.844726273995605%2C18.594001097453397+57.04751619889251\
%2C19.084102560907205+57.38153568787533%2C19.126352687067016+57.51792887758069%2C18.898202005804038\
+57.527003678430134%2C19.506603822505312+57.986831951833665%2C19.489703772041384+58.4098607116662%2\
C19.092552586139163+58.454097469115965%2C19.092552586139163+58.00922234044853%2C18.382750466654347+\
57.88814779931161%2C17.9686992302882+57.46343263752195%2C17.20819695941161+57.40430321908835%2C16.9\
5299378108794+57.365909645378736%2C16.741017592205917+57.30018179532556%2C16.885341805912827+57.603\
490785581556%2C16.921422859339554+57.968934636456424%2C17.109054049195077+58.21630220362434%2C17.25\
337826290199+58.64602619422499%2C17.67733064066603+58.60845833598157%2C19.066451197595036+59.144624\
9326259%2C19.908755784424883+59.41390842184913%2C19.97047130607586+59.59347345486486%2C19.283371831\
694968+60.05876192751174%2C19.054827807155824+60.211403624450725%2C19.1978804871813+60.637056514937\
655%2C18.319441860687476+60.636149631244365%2C17.805757245956226+60.70616385072543%2C17.66728574111\
563+60.730196083452626%2C17.457345072486337+61.20277592008492%2C17.636017981958073+61.7344663300663\
7%2C17.810224068693017+62.23589129340738%2C17.86203754676938+62.45936568442022%2C18.45612497076291+\
62.820664104781486%2C20.138794029375482+63.163776971079756%2C20.69953818653057+63.481897960994274%2\
C21.4387009391441+63.65435239635502%2C21.408114894208364+64.01178106292835%2C22.08791186403892+64.4\
5691794317739%2C21.771856066369686+64.85188805388998%2C22.495725796515345+65.08912860695378%2C22.90\
863740314773+65.13203758231046%2C23.714069919788678+65.33703869909073%2C24.133033533876585+65.53004\
214140537%2C24.179883311019854+65.66458856933014%2C24.16754955623186+65.69776069023371%2C24.1755482\
98670957+65.7278403993503%2C24.12891891588023+65.77762412821326%2C24.138300584862495+65.80092205426\
533%2C24.157063922827025+65.81608643908126%2C24.145597438515367+65.84105615161846%2C24.125270489053\
8+65.8496863271125%2C24.144555030850675+65.86119876685984%2C24.15706392282703+65.86119876685984%2C2\
4.15706392282703+65.86865777743552%2C24.11745243156858+65.93356606039367%2C24.074339291781712+65.96\
325722305681%2C23.96570263693073+66.12499352563663%2C23.914320888514744+66.17813017473608%2C23.7672\
74428879468+66.21396364112525%2C23.689589506807998+66.43233789509202%2C23.85867274893463+66.5431503\
0437087%2C23.916936440488232+66.60161450885823%2C23.95022997851886+66.7696178139654%2C24.0279149005\
9033+66.77071211836926%2C24.058433977118405+66.82427341956512%2C23.6018772788499+67.16744945690425%\
2C23.6536951151302+67.25776303021735%2C23.7789215528076+67.27445060525332%2C23.791876011877676+67.3\
9591901471326%2C23.748694481644087+67.4671807310342%2C23.480968994195862+67.47379889166325%2C23.640\
740656060128+67.65343594777602%2C23.54142313652288+67.85939601598903%2C23.71414925745722+67.9454903\
4629765%2C23.437787463962273+68.08451407195955%2C23.29528841419144+68.19384955775104%2C23.187334588\
607477+68.16817033014064%2C23.187334588607477+68.26431935703314%2C22.820291581622005+68.41573164506\
235%2C22.548247941150414+68.45699544517026%2C22.3130432721346+68.51636142503295%2C22.07765510518547\
5+68.4987611915873%2C21.99118598263274+68.57434485917426%2C21.746190135399974+68.61992371191332%2C2\
1.606878771287224+68.7003369026636%2C21.486782767741754+68.70382657910565%2C21.467567407174478+68.7\
822003070518%2C21.30904068249445+68.79089149826751%2C21.07845635568715+68.90010990987801%2C20.95836\
0352141675+68.92603507453188%2C20.99198723313441+69.01051220929742%2C20.622091542214356+69.07408670\
576356%2C19.997592323777898+69.05177070475449%2C20.23778433086884+68.92085247505727%2C20.2714112118\
6158+68.80826369376373%2C20.141707528032466+68.6846266102817%2C19.83906559909788+68.56907980600553%\
2C20.10808064703974+68.49347844364493%2C19.901515520941526+68.39817732069454%2C18.902316771443196+6\
8.53922120960533%2C18.652423533052048+68.52054160403026%2C18.38340848511019+68.60656855499036%2C18.\
07115887589196+68.55042496359118%2C18.03272815475741+68.36875821325506%2C18.099981916742877+68.1945\
5105815115%2C17.883809110361025+68.00282153564524%2C17.235290691215475+68.1445329288544%2C17.119998\
527811823+68.07109519872145%2C16.65402603405539+67.92712615463776%2C16.505106989659005+67.649281322\
83148%2C16.36579562554626+67.54675923817521%2C16.053546016328028+67.5302409325113%2C16.044914439650\
277+67.41931648524408%2C16.33841470808351+67.19595136591349%2C16.309064681240187+67.04553404126742%\
2C15.996887122997563+66.92138739595723%2C15.620673142551329+66.62147676194974%2C15.347609405092621+\
66.48284509748333%2C15.44099585413956+66.29475788463205%2C15.027427294074549+66.17863982432698%2C14\
.491122258119278+66.14521081627933%2C14.57261188674184+65.83954217298972%2C14.484561806211866+65.63\
120946682074%2C14.452543595110056+65.31705351552421%2C14.34047985625373+65.24005968470266%2C14.2977\
88908117989+65.12470811686794%2C13.610091783062513+64.59223363067966%2C13.860901103360003+64.497048\
82221283%2C14.087696765331136+64.46486329323128%2C14.123130051420205+64.2023427771424%2C13.94169352\
1843302+64.0217743013927%2C13.202606482243251+64.10929256759391%2C12.86374708141579+64.059149052090\
67%2C12.639619603703139+63.98668997100512%2C12.287419281583261+63.710334457094774%2C12.105982752006\
353+63.60377566738546%2C12.164682805693+63.47895043914954%2C11.932550775204895+63.26008861326167%2C\
12.170019174209967+63.00808586538648%2C12.039278145544255+62.891598580122746%2C12.092641830713932+6\
2.74167025427906%2C12.020600855734871+62.60816738647915%2C12.263405623256908+62.26121343211179%2C12\
.113987304781814+61.71107288821142%2C12.415492125990498+61.544977810428236%2C12.559574075948628+61.\
5551465959029%2C12.839733423089443+61.35241628117012%2C12.778365185144311+61.20624933072452%2C12.67\
6974183321922+61.148364340144354%2C12.650292340737083+61.071018906012796%2C12.20737375382875+61.034\
85958062632%2C12.175355542726944+61.02581330136968%2C12.298092018617204+60.88851113141288%2C12.3808\
0573063021+60.71670126372243%2C12.484864916711084+60.63699244491758%2C12.487533100969566+60.5911686\
9114885%2C12.572914997241053+60.52428035840245%2C12.586255918533473+60.412492061918556%2C12.4688558\
11160179+60.31749929768736%2C12.511546759295925+60.19571631058619%2C12.44751033709231+60.0614974141\
5368%2C12.164682805693014+59.90934420007886%2C11.919209853912491+59.9146947551593%2C11.804477930797\
683+59.83300503828363%2C11.908537116878556+59.77664291277904%2C11.905868932620072+59.71480357700216\
%2C11.865846168742813+59.70134509716834%2C11.831159773382522+59.65958940731977%2C11.65772779658107+\
59.62317913582825%2C11.740441508594072+59.44053445947637%2C11.796473378022233+59.25553437447812%2C1\
1.748446061369524+59.16128815626777%2C11.743109692852556+59.09831271863996%2C11.641718691030166+59.\
01462309872386%2C11.647055059547133+58.934860114451055%2C11.500304925330518+58.9031788218895%2C11.4\
8963218829658+59.00225818299083%2C11.35889115963087+59.13392175856134%2C11.121422760625801+59.08460\
707963016%2C11.070727259714603+59.00088402922154%2C10.611799567255368+58.8990443365781%2C10.8946270\
98654666+58.67920789029557%2C10.955995336599797+58.50958284348718%2C11.222813762448189+58.271845845\
261794%2C11.361154635131864+57.95545804184542%2C11.541453100372086+57.671912147221576%2C11.50711053\
5564426+57.45778675897438%2C12.517435834866681+56.75463762027667%2C12.603292246885836+56.5824436755\
8103%2C12.225524034001557+56.56115976906298%2C12.118203518977616+56.41896072560661%2C12.50353886049\
5686+56.14585246009915%2C12.653787581529206+56.026093982579766%2C12.615152196120587+55.905962874694\
61%2C12.765400917154105+55.763728031989416%2C12.888434515842775+55.62011379512285%2C12.680573141153\
63+55.503999043689845%2C12.629875244887984+55.339996317992075%2C12.845341304016978+55.2360644595551\
1%2C13.5855305894954+55.3024967606363%2C13.833950281197064+55.231728058034065%2C14.233971893326936+\
55.367302917260474))""",
    }
